@@ -350,7 +350,7 @@ absl::Status WeightedTargetLb::UpdateLocked(UpdateArgs args) {
   update_in_progress_ = false;
   if (config_->target_map().empty()) {
     absl::Status status = absl::UnavailableError(absl::StrCat(
-        "no children in weighted_target policy: ", args.resolution_note));
+        "no children in weighted_target policy (", args.resolution_note, ")"));
     channel_control_helper()->UpdateState(
         GRPC_CHANNEL_TRANSIENT_FAILURE, status,
         MakeRefCounted<TransientFailurePicker>(status));
