@@ -13,6 +13,7 @@
 // limitations under the License.
 #include "src/core/resolver/dns/event_engine/service_config_helper.h"
 
+#include <grpc/support/port_platform.h>
 #include <stdlib.h>
 
 #include <algorithm>
@@ -20,16 +21,13 @@
 
 #include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
-
-#include <grpc/support/port_platform.h>
-
-#include "src/core/lib/gprpp/status_helper.h"
-#include "src/core/lib/iomgr/gethostname.h"
+#include "src/core/util/gethostname.h"
 #include "src/core/util/json/json.h"
 #include "src/core/util/json/json_args.h"
 #include "src/core/util/json/json_object_loader.h"
 #include "src/core/util/json/json_reader.h"
 #include "src/core/util/json/json_writer.h"
+#include "src/core/util/status_helper.h"
 
 namespace grpc_core {
 
