@@ -36,6 +36,7 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <utility>
 
 #include "absl/log/check.h"
@@ -43,9 +44,8 @@
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_join.h"
-#include "absl/types/optional.h"
+#include "src/core/credentials/call/json_util.h"
 #include "src/core/lib/event_engine/default_event_engine.h"
-#include "src/core/lib/security/util/json_util.h"
 #include "src/core/util/env.h"
 #include "src/core/util/json/json.h"
 #include "src/core/util/json/json_reader.h"
@@ -348,7 +348,6 @@ class MetadataCredentialsPluginWrapper final : private internal::GrpcLibrary {
   static void Destroy(void* wrapper) {
     if (wrapper == nullptr) return;
     grpc_event_engine::experimental::GetDefaultEventEngine()->Run([wrapper] {
-      grpc_core::ApplicationCallbackExecCtx callback_exec_ctx;
       grpc_core::ExecCtx exec_ctx;
       delete static_cast<MetadataCredentialsPluginWrapper*>(wrapper);
     });
