@@ -21,6 +21,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <memory>
+#include <optional>
 #include <string>
 #include <tuple>
 #include <vector>
@@ -31,7 +32,6 @@
 #include "absl/strings/str_format.h"
 #include "absl/strings/str_join.h"
 #include "absl/strings/string_view.h"
-#include "absl/types/optional.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
 #include "src/core/config/config_vars.h"
@@ -116,7 +116,7 @@ class EventEngineDNSTest : public EventEngineTest {
     std::string dns_resolver_path = kDNSResolverRelPath;
     std::string tcp_connect_path = kTCPConnectRelPath;
     std::string health_check_path = kHealthCheckRelPath;
-    absl::optional<std::string> runfile_dir = grpc::GetGrpcTestRunFileDir();
+    std::optional<std::string> runfile_dir = grpc::GetGrpcTestRunFileDir();
     if (runfile_dir.has_value()) {
       test_records_path = absl::StrJoin({*runfile_dir, test_records_path}, "/");
       dns_server_path = absl::StrJoin({*runfile_dir, dns_server_path}, "/");
