@@ -16,8 +16,7 @@
 //
 //
 
-#include <gtest/gtest.h>
-
+#include "gtest/gtest.h"
 #include "src/core/lib/iomgr/port.h"
 #include "test/core/test_util/test_config.h"
 
@@ -27,6 +26,10 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <grpc/grpc.h>
+#include <grpc/support/alloc.h>
+#include <grpc/support/sync.h>
+#include <grpc/support/time.h>
 #include <netinet/in.h>
 #include <poll.h>
 #include <stdio.h>
@@ -37,18 +40,10 @@
 #include <unistd.h>
 
 #include "absl/log/log.h"
-#include "absl/strings/str_format.h"
-
-#include <grpc/grpc.h>
-#include <grpc/support/alloc.h>
-#include <grpc/support/sync.h>
-#include <grpc/support/time.h>
-
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/strerror.h"
+#include "src/core/lib/experiments/experiments.h"
 #include "src/core/lib/iomgr/ev_posix.h"
-#include "src/core/lib/iomgr/iomgr.h"
 #include "src/core/lib/iomgr/socket_utils_posix.h"
+#include "src/core/util/strerror.h"
 
 static gpr_mu* g_mu;
 static grpc_pollset* g_pollset;
@@ -516,6 +511,9 @@ static void destroy_pollset(void* p, grpc_error_handle /*error*/) {
 TEST(FdPosixTest, MainTest) {
   grpc_closure destroyed;
   grpc_init();
+  if (grpc_core::IsPollsetAlternativeEnabled()) {
+    GTEST_SKIP() << "Skipping test since we're using pollset alternative";
+  }
   {
     grpc_core::ExecCtx exec_ctx;
     g_pollset = static_cast<grpc_pollset*>(gpr_zalloc(grpc_pollset_size()));
