@@ -36,33 +36,30 @@
 #include "absl/strings/str_format.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
+#include "src/core/call/metadata_batch.h"
 #include "src/core/config/core_configuration.h"
 #include "src/core/ext/transport/chaotic_good/chaotic_good_frame.pb.h"
+#include "src/core/ext/transport/chaotic_good/frame.h"
 #include "src/core/lib/promise/if.h"
 #include "src/core/lib/promise/loop.h"
 #include "src/core/lib/promise/seq.h"
 #include "src/core/lib/resource_quota/arena.h"
 #include "src/core/lib/slice/slice_buffer.h"
-#include "src/core/lib/transport/metadata_batch.h"
-#include "test/core/transport/chaotic_good/transport_test.h"
+#include "test/core/transport/chaotic_good/mock_frame_transport.h"
+#include "test/core/transport/chaotic_good/transport_test_helper.h"
 #include "test/core/transport/util/mock_promise_endpoint.h"
+#include "test/core/transport/util/transport_test.h"
 
 using testing::MockFunction;
-using testing::Return;
 using testing::StrictMock;
 
 using EventEngineSlice = grpc_event_engine::experimental::Slice;
+using grpc_core::util::testing::TransportTest;
 
 namespace grpc_core {
 namespace chaotic_good {
 namespace testing {
 
-class MockClientConnectionFactory : public ClientConnectionFactory {
- public:
-  MOCK_METHOD(PendingConnection, Connect, (absl::string_view), (override));
-  void Orphaned() final {}
-};
-
 ClientMetadataHandle TestInitialMetadata() {
   auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   md->Set(HttpPathMetadata(), Slice::FromStaticString("/demo.Service/Step"));
@@ -101,63 +98,20 @@ ChannelArgs MakeChannelArgs(
           std::move(event_engine));
 }
 
-template <typename... PromiseEndpoints>
-Config MakeConfig(const ChannelArgs& channel_args,
-                  PromiseEndpoints... promise_endpoints) {
-  Config config(channel_args);
-  auto name_endpoint = [i = 0]() mutable { return absl::StrCat(++i); };
-  std::vector<int> this_is_only_here_to_unpack_the_following_statement{
-      (config.ServerAddPendingDataEndpoint(
-           ImmediateConnection(name_endpoint(), std::move(promise_endpoints))),
-       0)...};
-  return config;
-}
-
 TEST_F(TransportTest, AddOneStream) {
-  MockPromiseEndpoint control_endpoint(1000);
-  MockPromiseEndpoint data_endpoint(1001);
-  auto client_connection_factory =
-      MakeRefCounted<StrictMock<MockClientConnectionFactory>>();
+  auto owned_frame_transport = MakeOrphanable<MockFrameTransport>();
+  auto* frame_transport = owned_frame_transport.get();
   static const std::string many_as(1024 * 1024, 'a');
-  const auto server_initial_metadata =
-      EncodeProto<chaotic_good_frame::ServerMetadata>("message: 'hello'");
-  const auto server_trailing_metadata =
-      EncodeProto<chaotic_good_frame::ServerMetadata>("status: 0");
-  const auto client_initial_metadata =
-      EncodeProto<chaotic_good_frame::ClientMetadata>(
-          "path: '/demo.Service/Step'");
-  control_endpoint.ExpectRead(
-      {SerializedFrameHeader(FrameType::kServerInitialMetadata, 0, 1,
-                             server_initial_metadata.length()),
-       server_initial_metadata.Copy(),
-       SerializedFrameHeader(FrameType::kMessage, 1, 1, many_as.length()),
-       SerializedFrameHeader(FrameType::kServerTrailingMetadata, 0, 1,
-                             server_trailing_metadata.length()),
-       server_trailing_metadata.Copy()},
-      event_engine().get());
-  data_endpoint.ExpectRead(
-      {EventEngineSlice::FromCopiedString(many_as), Zeros(56)}, nullptr);
-  EXPECT_CALL(*control_endpoint.endpoint, Read)
-      .InSequence(control_endpoint.read_sequence)
-      .WillOnce(Return(false));
   auto channel_args = MakeChannelArgs(event_engine());
   auto transport = MakeOrphanable<ChaoticGoodClientTransport>(
-      channel_args, std::move(control_endpoint.promise_endpoint),
-      MakeConfig(channel_args, std::move(data_endpoint.promise_endpoint)),
-      client_connection_factory);
+      channel_args, std::move(owned_frame_transport), MessageChunker(0, 1));
   auto call = MakeCall(TestInitialMetadata());
   StrictMock<MockFunction<void()>> on_done;
   EXPECT_CALL(on_done, Call());
-  control_endpoint.ExpectWrite(
-      {SerializedFrameHeader(FrameType::kClientInitialMetadata, 0, 1,
-                             client_initial_metadata.length()),
-       client_initial_metadata.Copy()},
-      nullptr);
-  control_endpoint.ExpectWrite(
-      {SerializedFrameHeader(FrameType::kMessage, 0, 1, 1),
-       EventEngineSlice::FromCopiedString("0"),
-       SerializedFrameHeader(FrameType::kClientEndOfStream, 0, 1, 0)},
-      nullptr);
+  frame_transport->ExpectWrite(MakeProtoFrame<ClientInitialMetadataFrame>(
+      1, "path: '/demo.Service/Step'"));
+  frame_transport->ExpectWrite(MakeMessageFrame(1, "0"));
+  frame_transport->ExpectWrite(ClientEndOfStream(1));
   transport->StartCall(call.handler.StartCall());
   call.initiator.SpawnGuarded("test-send",
                               [initiator = call.initiator]() mutable {
@@ -195,58 +149,30 @@ TEST_F(TransportTest, AddOneStream) {
               on_done.Call();
             });
       });
+  frame_transport->Read(
+      MakeProtoFrame<ServerInitialMetadataFrame>(1, "message: 'hello'"));
+  frame_transport->Read(MakeMessageFrame(1, many_as));
+  frame_transport->Read(
+      MakeProtoFrame<ServerTrailingMetadataFrame>(1, "status: 0"));
   // Wait until ClientTransport's internal activities to finish.
   event_engine()->TickUntilIdle();
   event_engine()->UnsetGlobalHooks();
 }
 
 TEST_F(TransportTest, AddOneStreamMultipleMessages) {
-  MockPromiseEndpoint control_endpoint(1000);
-  MockPromiseEndpoint data_endpoint(1001);
-  auto client_connection_factory =
-      MakeRefCounted<StrictMock<MockClientConnectionFactory>>();
-  const auto server_initial_metadata =
-      EncodeProto<chaotic_good_frame::ServerMetadata>("");
-  const auto server_trailing_metadata =
-      EncodeProto<chaotic_good_frame::ServerMetadata>("status: 0");
-  const auto client_initial_metadata =
-      EncodeProto<chaotic_good_frame::ClientMetadata>(
-          "path: '/demo.Service/Step'");
-  control_endpoint.ExpectRead(
-      {SerializedFrameHeader(FrameType::kServerInitialMetadata, 0, 1,
-                             server_initial_metadata.length()),
-       server_initial_metadata.Copy(),
-       SerializedFrameHeader(FrameType::kMessage, 0, 1, 8),
-       EventEngineSlice::FromCopiedString("12345678"),
-       SerializedFrameHeader(FrameType::kMessage, 0, 1, 8),
-       EventEngineSlice::FromCopiedString("87654321"),
-       SerializedFrameHeader(FrameType::kServerTrailingMetadata, 0, 1,
-                             server_trailing_metadata.length()),
-       server_trailing_metadata.Copy()},
-      event_engine().get());
-  EXPECT_CALL(*control_endpoint.endpoint, Read)
-      .InSequence(control_endpoint.read_sequence)
-      .WillOnce(Return(false));
+  auto owned_frame_transport = MakeOrphanable<MockFrameTransport>();
+  auto* frame_transport = owned_frame_transport.get();
   auto channel_args = MakeChannelArgs(event_engine());
   auto transport = MakeOrphanable<ChaoticGoodClientTransport>(
-      channel_args, std::move(control_endpoint.promise_endpoint),
-      MakeConfig(channel_args, std::move(data_endpoint.promise_endpoint)),
-      client_connection_factory);
+      channel_args, std::move(owned_frame_transport), MessageChunker(0, 1));
   auto call = MakeCall(TestInitialMetadata());
   StrictMock<MockFunction<void()>> on_done;
   EXPECT_CALL(on_done, Call());
-  control_endpoint.ExpectWrite(
-      {SerializedFrameHeader(FrameType::kClientInitialMetadata, 0, 1,
-                             client_initial_metadata.length()),
-       client_initial_metadata.Copy()},
-      nullptr);
-  control_endpoint.ExpectWrite(
-      {SerializedFrameHeader(FrameType::kMessage, 0, 1, 1),
-       EventEngineSlice::FromCopiedString("0"),
-       SerializedFrameHeader(FrameType::kMessage, 0, 1, 1),
-       EventEngineSlice::FromCopiedString("1"),
-       SerializedFrameHeader(FrameType::kClientEndOfStream, 0, 1, 0)},
-      nullptr);
+  frame_transport->ExpectWrite(MakeProtoFrame<ClientInitialMetadataFrame>(
+      1, "path: '/demo.Service/Step'"));
+  frame_transport->ExpectWrite(MakeMessageFrame(1, "0"));
+  frame_transport->ExpectWrite(MakeMessageFrame(1, "1"));
+  frame_transport->ExpectWrite(ClientEndOfStream(1));
   transport->StartCall(call.handler.StartCall());
   call.initiator.SpawnGuarded("test-send",
                               [initiator = call.initiator]() mutable {
@@ -283,11 +209,52 @@ TEST_F(TransportTest, AddOneStreamMultipleMessages) {
               on_done.Call();
             });
       });
+  frame_transport->Read(MakeProtoFrame<ServerInitialMetadataFrame>(1, ""));
+  frame_transport->Read(MakeMessageFrame(1, "12345678"));
+  frame_transport->Read(MakeMessageFrame(1, "87654321"));
+  frame_transport->Read(
+      MakeProtoFrame<ServerTrailingMetadataFrame>(1, "status: 0"));
   // Wait until ClientTransport's internal activities to finish.
   event_engine()->TickUntilIdle();
   event_engine()->UnsetGlobalHooks();
 }
 
+TEST_F(TransportTest, CheckFailure) {
+  auto owned_frame_transport = MakeOrphanable<MockFrameTransport>();
+  auto* frame_transport = owned_frame_transport.get();
+  auto transport = MakeOrphanable<ChaoticGoodClientTransport>(
+      MakeChannelArgs(event_engine()), std::move(owned_frame_transport),
+      MessageChunker(0, 1));
+  frame_transport->Close();
+  auto call = MakeCall(TestInitialMetadata());
+  transport->StartCall(call.handler.StartCall());
+  call.initiator.SpawnGuarded("test-send",
+                              [initiator = call.initiator]() mutable {
+                                return SendClientToServerMessages(initiator, 1);
+                              });
+  StrictMock<MockFunction<void()>> on_done;
+  EXPECT_CALL(on_done, Call());
+  call.initiator.SpawnInfallible(
+      "test-read", [&on_done, initiator = call.initiator]() mutable {
+        return Seq(
+            initiator.PullServerInitialMetadata(),
+            [](ValueOrFailure<absl::optional<ServerMetadataHandle>> md) {
+              EXPECT_TRUE(md.ok());
+            },
+            initiator.PullServerTrailingMetadata(),
+            [&on_done](ServerMetadataHandle md) {
+              EXPECT_EQ(md->get(GrpcStatusMetadata()).value(),
+                        GRPC_STATUS_UNAVAILABLE);
+              on_done.Call();
+            });
+      });
+  // Wait until ClientTransport's internal activities to finish.
+  event_engine()->TickUntilIdle();
+  transport.reset();
+  event_engine()->TickUntilIdle();
+  event_engine()->UnsetGlobalHooks();
+}
+
 }  // namespace testing
 }  // namespace chaotic_good
 }  // namespace grpc_core
