@@ -29,12 +29,12 @@
 #include "absl/log/check.h"
 #include "absl/log/log.h"
 #include "absl/random/random.h"
+#include "src/core/call/metadata_batch.h"
 #include "src/core/ext/transport/chttp2/transport/hpack_encoder.h"
 #include "src/core/ext/transport/chttp2/transport/hpack_parser.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/lib/slice/slice_string_helpers.h"
-#include "src/core/lib/transport/metadata_batch.h"
 #include "src/core/lib/transport/timeout_encoding.h"
 #include "src/core/util/crash.h"
 #include "src/core/util/time.h"
@@ -63,18 +63,16 @@ class FakeCallTracer final : public CallTracerInterface {
       grpc_metadata_batch* send_initial_metadata) override {}
   void RecordSendTrailingMetadata(
       grpc_metadata_batch* send_trailing_metadata) override {}
-  void RecordSendMessage(const SliceBuffer& send_message) override {}
+  void RecordSendMessage(const Message& send_message) override {}
   void RecordSendCompressedMessage(
-      const SliceBuffer& send_compressed_message) override {}
+      const Message& send_compressed_message) override {}
   void RecordReceivedInitialMetadata(
       grpc_metadata_batch* recv_initial_metadata) override {}
-  void RecordReceivedMessage(const SliceBuffer& recv_message) override {}
+  void RecordReceivedMessage(const Message& recv_message) override {}
   void RecordReceivedDecompressedMessage(
-      const SliceBuffer& recv_decompressed_message) override {}
+      const Message& recv_decompressed_message) override {}
   void RecordCancel(grpc_error_handle cancel_error) override {}
-  std::shared_ptr<TcpTracerInterface> StartNewTcpTrace() override {
-    return nullptr;
-  }
+  std::shared_ptr<TcpCallTracer> StartNewTcpTrace() override { return nullptr; }
   void RecordAnnotation(absl::string_view annotation) override {}
   void RecordAnnotation(const Annotation& annotation) override {}
   std::string TraceId() override { return ""; }
